/* ==================================================================
   File:        error.cpp
   Author:      Steve Monks
   Description:
	Debug and error handling stuff

 	$Id: error.cpp,v 1.4 2007/01/06 21:50:15 Steve Exp $
	$Log: error.cpp,v $
	Revision 1.4  2007/01/06 21:50:15  Steve
	Changes for version 3.61
	Fixed issue where log files could be created in different locations depending
	on how Winamp was launched.
	
	Revision 1.3  2006/04/09 19:51:53  Steve
	Version 3.6
	Added logarithmic volume control
	Added diagnostic logging
	
	Revision 1.2  2006/02/11 14:06:00  Steve
	Fix for Gapless Mode crashes when format changes.
	Fix for Gapless Mode lockup when track changed while paused
	Added format info to status window
	Fixes to Release CFG project setting (removed debug info and _DEBUG define)
	
	Revision 1.1.1.1  2006/01/02 12:16:11  Steve
	Initial import
	
	Revision 1.2  2005/12/24 01:29:15  Steve
	Added Volume control for 16 bit audio
	Added option to release filter at end of each track or when stopped
	Added tool tips to config dialog
	Fixed Filter selection to use friendly names rather than device paths
	
	Revision 1.1  2005/12/10 18:59:36  Steve
	Fixed crash on pause
	Fixed pause interference with start of new track
	Fixed problem with next track not playing when WAV files end
	
================================================================== */

#include "windows.h"
#include "shlwapi.h"
#include "stdarg.h"
#include "string.h"
#include "stdio.h"
#include "error.h"

CRITICAL_SECTION cs_error;	// protects debug message and file logging from simultaneous multiple thread access
static HWND main_wnd;
static char msg_title[512] = "";
static int msg_loggable;

static char kslog_name[MAX_PATH+1];

static int logging_enabled = FALSE;

void DoDebugMsgBox(char *fmt, ...)
{
	char text[2048];
	va_list argptr;

	va_start(argptr,fmt);
	vsprintf(text,fmt,argptr);
	va_end(argptr);

	MessageBox(main_wnd, text, msg_title, MB_OK);
}



debugmsgfunc *InitDebugMsgBox(char *file, int line)
{
	sprintf(msg_title,"%s (%d)", file, line);
	return(&DoDebugMsgBox);
}



void DoDebugMsg(char *fmt, ...)
{
	char text[4096];
	va_list argptr;

	va_start(argptr,fmt);
	vsprintf(text,fmt,argptr);
	va_end(argptr);

	strcat(text,"\r\n");

	if (msg_loggable && logging_enabled) {
		FILE *fh;

		fh = fopen(kslog_name,"a+b");
		if (fh) {
			fprintf(fh,text);
			fclose(fh);
		}
	}

#ifdef _DEBUG
	OutputDebugString(msg_title);
	OutputDebugString(text);
#endif

	LeaveCriticalSection(&cs_error);	// Entered in InitDebugMsg

}



debugmsgfunc *InitDebugMsg(int loggable, char *file, int line)
{
	EnterCriticalSection(&cs_error);	// leaves it in DoDebugMsg

	msg_loggable = loggable;
	sprintf(msg_title,"%s (%d)", file, line);
	return(&DoDebugMsg);
}


// Note this functions is always called even if logging is disabled, thereby ensuring that
// the logfile gets cleared every time the program is launched.

static void InitLogging(void)
{
	FILE *fh;

	// Create the log file name by getting the path from the application
	GetModuleFileName(NULL,kslog_name,sizeof(kslog_name));
	PathRemoveFileSpec(kslog_name);
	strcat(kslog_name,"\\kernel streaming log.txt");

	fh = fopen(kslog_name,"wb");
	if (fh) {
		fclose(fh);
	}
}

void EnableLogging(int enable)
{
	logging_enabled = enable;
}

						
void DebugMsgInit(HWND wnd)
{
	InitializeCriticalSection(&cs_error);
	main_wnd = wnd;
	InitLogging();

}



void DebugMsgClose(void)
{
	DeleteCriticalSection(&cs_error);
}

